@@ -6,6 +6,8 @@ use std::io::ErrorKind;
 use std::marker::PhantomData;
 use std::pin::Pin;
 use std::sync::Arc;
+use std::sync::atomic::Ordering::SeqCst;
+use std::sync::atomic::AtomicI32;
 use std::time::Duration;
 use std::fmt;
 
@@ -13,7 +15,7 @@ use async_channel::bounded;
 use async_channel::Receiver;
 use async_channel::Sender;
 use async_lock::Mutex;
-use bytes::BytesMut;
+use bytes::{Bytes};
 use event_listener::Event;
 use fluvio_future::net::ConnectionFd;
 use futures_util::stream::{Stream, StreamExt};
@@ -34,29 +36,21 @@ use crate::FluvioStream;
 
 pub type SharedMultiplexerSocket = Arc<MultiplexerSocket>;
 
-type SharedMsg = (Arc<Mutex<Option<BytesMut>>>, Arc<Event>);
+type SharedMsg = (Arc<Mutex<Option<Bytes>>>, Arc<Event>);
 
 /// Handle different way to multiplex
 enum SharedSender {
     /// Serial socket
     Serial(SharedMsg),
     /// Batch Socket
-    Queue(Sender<Option<BytesMut>>),
+    Queue(Sender<Option<Bytes>>),
 }
 
 type Senders = Arc<Mutex<HashMap<i32, SharedSender>>>;
 
-async fn correlation_id(counter: Arc<Mutex<i32>>) -> i32 {
-    let mut guard = counter.lock().await;
-    let current_value = *guard;
-    // update to new
-    *guard = current_value + 1;
-    current_value
-}
-
 /// Socket that can multiplex connections
 pub struct MultiplexerSocket {
-    correlation_id_counter: Arc<Mutex<i32>>,
+    correlation_id_counter: AtomicI32,
     senders: Senders,
     sink: ExclusiveFlvSink,
     terminate: Arc<Event>,
@@ -90,7 +84,7 @@ impl MultiplexerSocket {
         let (sink, stream) = socket.split();
 
         let multiplexer = Self {
-            correlation_id_counter: Arc::new(Mutex::new(1)),
+            correlation_id_counter: AtomicI32::new(1),
             senders: Arc::new(Mutex::new(HashMap::new())),
             sink: ExclusiveFlvSink::new(sink),
             terminate: Arc::new(Event::new()),
@@ -107,9 +101,8 @@ impl MultiplexerSocket {
     }
 
     /// get next available correlation to use
-    //  use lock to ensure update happens in orderly manner
-    async fn next_correlation_id(&self) -> i32 {
-        correlation_id(self.correlation_id_counter.clone()).await
+    fn next_correlation_id(&self) -> i32 {
+        self.correlation_id_counter.fetch_add(1, SeqCst)
     }
 
     /// create socket to perform request and response
@@ -131,7 +124,7 @@ impl MultiplexerSocket {
             wait_time
         });
 
-        let correlation_id = self.next_correlation_id().await;
+        let correlation_id = self.next_correlation_id();
         let bytes_lock: SharedMsg = (Arc::new(Mutex::new(None)), Arc::new(Event::new()));
 
         req_msg.header.set_correlation_id(correlation_id);
@@ -162,7 +155,7 @@ impl MultiplexerSocket {
 
                 Err(IoError::new(
                     ErrorKind::TimedOut,
-                    format!("time out in serial: {} request: {}", R::API_KEY, correlation_id),
+                    format!("Timed out waiting for response. API_KEY={}, CorrelationId={}", R::API_KEY, correlation_id),
                 ).into())
             },
 
@@ -214,7 +207,7 @@ impl MultiplexerSocket {
     where
         R: Request,
     {
-        let correlation_id = self.next_correlation_id().await;
+        let correlation_id = self.next_correlation_id();
 
         req_msg.header.set_correlation_id(correlation_id);
 
@@ -257,7 +250,7 @@ impl MultiplexerSocket {
 #[pin_project(PinnedDrop)]
 pub struct AsyncResponse<R> {
     #[pin]
-    receiver: Receiver<Option<BytesMut>>,
+    receiver: Receiver<Option<Bytes>>,
     header: RequestHeader,
     correlation_id: i32,
     data: PhantomData<R>,
@@ -291,29 +284,33 @@ impl<R: Request> Stream for AsyncResponse<R> {
             Poll::Ready(next) => next,
         };
 
-        let bytes = if let Some(bytes) = next {
-            bytes
-        } else {
-            return Poll::Ready(None);
-        };
+        if let Some(bytes) = next {
+            if let Some(msg) = bytes {
+                use bytes::Buf;
+                let response_len = msg.len();
+                debug!(
+                    response_len,
+                    remaining = msg.remaining(),
+                    version = this.header.api_version(),
+                    "response len>>>"
+                );
 
-        let bytes = if let Some(bytes) = bytes {
-            bytes
-        } else {
-            return Poll::Ready(Some(Err(SocketError::SocketClosed)));
-        };
-
-        let mut cursor = Cursor::new(&bytes);
-        let response = R::Response::decode_from(&mut cursor, this.header.api_version());
-
-        let value = match response {
-            Ok(value) => {
-                trace!("Received response bytes: {},  {:#?}", bytes.len(), &value,);
-                Some(Ok(value))
+                let mut cursor = Cursor::new(msg);
+                let response = R::Response::decode_from(&mut cursor, this.header.api_version());
+                let value = match response {
+                    Ok(value) => {
+                        trace!("Received response bytes: {},  {:#?}", response_len, &value,);
+                        Some(Ok(value))
+                    }
+                    Err(e) => Some(Err(e.into())),
+                };
+                Poll::Ready(value)
+            } else {
+                Poll::Ready(Some(Err(SocketError::SocketClosed)))
             }
-            Err(e) => Some(Err(e.into())),
-        };
-        Poll::Ready(value)
+        } else {
+            Poll::Ready(None)
+        }
     }
 }
 
@@ -357,9 +354,10 @@ impl MultiPlexingResponseDispatcher {
                             let mut correlation_id: i32 = 0;
                             match correlation_id.decode(&mut msg, 0) {
                                 Ok(_) => {
-                                    trace!(correlation_id,len = msg.len(), "received frame");
+                                    use bytes::Buf;
+                                    debug!(correlation_id,len = msg.len(), remaining = msg.remaining(), "received frame");
 
-                                    if let Err(err) = self.send(correlation_id, msg).await {
+                                    if let Err(err) = self.send(correlation_id, msg.freeze()).await {
                                         error!("error sending to socket, {}", err)
                                     }
                                 }
@@ -408,7 +406,7 @@ impl MultiPlexingResponseDispatcher {
 
     /// send message to correct receiver
     #[instrument(skip(self, msg),fields( msg = msg.len()))]
-    async fn send(&mut self, correlation_id: i32, msg: BytesMut) -> Result<(), SocketError> {
+    async fn send(&mut self, correlation_id: i32, msg: Bytes) -> Result<(), SocketError> {
         let mut senders = self.senders.lock().await;
         if let Some(sender) = senders.get_mut(&correlation_id) {
             match sender {
