@@ -1,10 +1,10 @@
 use std::convert::TryInto;
 
-use fluvio_protocol::derive::{Decode, Encode};
+use fluvio_protocol::derive::{Decoder, Encoder};
 
-#[fluvio(encode_discriminant)]
-#[derive(Encode, Decode, PartialEq, Debug, Clone, Copy)]
 #[repr(u16)]
+#[derive(Encoder, Decoder, PartialEq, Debug, Clone, Copy)]
+#[fluvio(encode_discriminant)]
 pub enum TestKafkaApiEnum {
     Echo = 1000,
     Status = 1001,
